@@ -6,9 +6,12 @@ from docx.oxml.table import CT_Tbl
 from docx.oxml.text.paragraph import CT_P
 from docx.table import _Cell, Table
 from docx.text.paragraph import Paragraph
+from gensim.similarities import SparseMatrixSimilarity
+from gensim.utils import simple_preprocess
+from gensim.corpora.dictionary import Dictionary
+from gensim.models.tfidfmodel import TfidfModel
 
 from srsparser import configs
-from srsparser.nlp import strings_similarity
 from srsparser.sections_tree import SectionsTree
 
 
@@ -25,6 +28,24 @@ class Parser:
         """
         self.sections_tree = SectionsTree(sections_tree_template)
 
+        # создаём список, состоящий из элементов,
+        # каждый из которых представлен списком слов заголовков листовых разделов
+        self.leaf_section_headings = self.sections_tree.get_leaf_sections_names()
+        gen_docs = [[word for word in simple_preprocess(name, min_len=1, max_len=50)]
+                    for name in self.leaf_section_headings]
+
+        # сопоставляем каждое слово с уникальным идентификатором (образование словаря)
+        self.dictionary = Dictionary(gen_docs)
+
+        # создаём корпус (набор слов), который содержит идентификатор слова и его частоту в каждом документе (разделе)
+        # (перечисляет, сколько раз слово встретилось в предложении)
+        corpus = [self.dictionary.doc2bow(gen_doc) for gen_doc in gen_docs]
+
+        # TF-IDF веса слов для всех документов (разделов)
+        self.tf_idf = TfidfModel(corpus, smartirs="ltc")
+
+        self.similarity = SparseMatrixSimilarity(self.tf_idf[corpus], num_features=len(self.dictionary))
+
     def parse_docx(self, path: str) -> dict:
         """
         Возвращает заполненную в соответствии с содержимым текстового документа с ТЗ структуру дерева разделов.
@@ -94,15 +115,17 @@ class Parser:
         Заполняет структуру дерева разделов, выраженную уровнями вложенности разделов.
         """
         for heading, text in headings_with_texts.items():
-            max_ratio = 0
-            text_parent = None
-            for section in self.sections_tree.get_leaf_sections():
-                ratio = strings_similarity(section.name, heading)
-                if ratio >= max_ratio:
-                    max_ratio = ratio
-                    text_parent = section
-            if text_parent is not None:
-                text_parent.text = text
+            query_doc = [word for word in simple_preprocess(heading, min_len=1, max_len=50)]
+            query_doc_bow = self.dictionary.doc2bow(query_doc)
+            query_doc_tf_idf = self.tf_idf[query_doc_bow]
+
+            max_idx = 0
+            for i in range(1, len(self.similarity[query_doc_tf_idf])):
+                if self.similarity[query_doc_tf_idf][i] > self.similarity[query_doc_tf_idf][max_idx]:
+                    max_idx = i
+
+            text_parent = self.sections_tree.get_leaf_sections()[max_idx]
+            text_parent.text = text
 
     def iter_paragraphs(self, parent):
         if isinstance(parent, DocumentWithTable):
@@ -128,10 +151,10 @@ class Parser:
 
         :return: True — да, иначе — False.
         """
-        for section in self.sections_tree.get_leaf_sections():
-            if strings_similarity(section.name, p_text) >= configs.MIN_SIMILARITY_RATIO:
-                return True
-        return False
+        query_doc = [word for word in simple_preprocess(p_text, min_len=1, max_len=50)]
+        query_doc_bow = self.dictionary.doc2bow(query_doc)
+        query_doc_tf_idf = self.tf_idf[query_doc_bow]
+        return max(self.similarity[query_doc_tf_idf]) >= configs.MIN_SIMILARITY_RATIO
 
     @staticmethod
     def is_table_element(paragraph: Paragraph, doc: Document) -> bool:
